@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2006 - 2016, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2017, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -48,33 +48,8 @@ int cmdDiagnostics ()
 
   std::cout << "\n"
             << bold.colorize (PACKAGE_STRING)
-            << "\n";
-
-  std::cout << "   " << "Platform: "
-            <<
-#if defined (DARWIN)
-               "Darwin"
-#elif defined (SOLARIS)
-               "Solaris"
-#elif defined (CYGWIN)
-               "Cygwin"
-#elif defined (HAIKU)
-               "Haiku"
-#elif defined (OPENBSD)
-               "OpenBSD"
-#elif defined (FREEBSD)
-               "FreeBSD"
-#elif defined (NETBSD)
-               "NetBSD"
-#elif defined (LINUX)
-               "Linux"
-#elif defined (KFREEBSD)
-TRING_CMD_DIAG_COMPILER
-#elif defined (GNUHURD)
-               "GNU/Hurd"
-#else
-               "Unknown"
-#endif
+            << "\n"
+            << "   " << "Platform: " << osName ()
             << "\n\n";
 
   // Compiler.
@@ -114,18 +89,8 @@ TRING_CMD_DIAG_COMPILER
             << "\n";
 
   // Compiler compliance level.
-  std::string compliance = "non-compliant";
-#ifdef __cplusplus
-  int level = __cplusplus;
-  if (level == 199711)
-    compliance = "C++98/03";
-  else if (level == 201103)
-    compliance = "C++11";
-  else
-    compliance = format (level);
-#endif
   std::cout << " Compliance: "
-            << compliance
+            << cppCompliance ()
             << "\n\n";
 
   std::cout << bold.colorize ("Build Features")
@@ -161,7 +126,7 @@ TRING_CMD_DIAG_COMPILER
   std::cout << bold.colorize ("Configuration")
             << "\n";
 
-  char* env = getenv ("TASKRC");
+  auto env = getenv ("TASKRC");
   std::cout << "     TASKRC: "
             << (env ? env : "")
             << "\n";
@@ -175,12 +140,9 @@ TRING_CMD_DIAG_COMPILER
   std::string path (getenv ("PATH"));
   std::cout << "       PATH: " << path << "\n";
 
-  std::vector <std::string> paths = split (path, ':');
-
-  std::vector <std::string>::iterator i;
-  for (i = paths.begin (); i != paths.end (); ++i)
+  for (const auto& i : split (path, ':'))
   {
-    File task (*i + "/task");
+    File task (i + "/task");
     if (task.exists ())
     {
       std::string input;
@@ -188,8 +150,8 @@ TRING_CMD_DIAG_COMPILER
       execute ("task", {"--version"}, input, output);
 
       std::cout << "Taskwarrior: "
-                << (*i + "/task")
-                << " "
+                << i
+                << "/task "
                 << output; // Still has \n
     }
   }
