@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2013 - 2016, Paul Beckingham, Federico Hernandez.
+// Copyright 2013 - 2017, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -295,7 +295,7 @@ bool Lexer::isPunctuation (int c)
 // characters.
 std::string Lexer::dequote (const std::string& input, const std::string& quotes)
 {
-  if (input.length ())
+  if (input.length () > 1)
   {
     int quote = input[0];
     if (quotes.find (quote) != std::string::npos)
@@ -490,9 +490,9 @@ bool Lexer::isUUID (std::string& token, Lexer::Type& type, bool endBoundary)
         break;
     }
 
-    if (i >= uuid_min_length              &&
-        (! endBoundary                    ||
-         ! _text[marker + i]              ||
+    if (i >= uuid_min_length                   &&
+        (! endBoundary                         ||
+         ! _text[marker + i]                   ||
          unicodeWhitespace (_text[marker + i]) ||
          isSingleCharOperator (_text[marker + i])))
     {
@@ -546,9 +546,9 @@ bool Lexer::isWord (std::string& token, Lexer::Type& type)
   {
     std::size_t marker = _cursor;
 
-    while (_text[marker]                  &&
+    while (_text[marker] &&
            ! unicodeWhitespace (_text[marker]) &&
-           ! isSingleCharOperator (_text[marker]))
+           (! _enableOperator || ! isSingleCharOperator (_text[marker])))
       utf8_next_char (_text, marker);
 
     if (marker > _cursor)
