@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2006 - 2016, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2017, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -29,7 +29,9 @@
 #include <vector>
 #include <string>
 #include <cstring>
+#include <cstdio>
 #include <stdlib.h>
+#include <unistd.h>
 #include <shared.h>
 
 #ifdef HAVE_READLINE
@@ -67,6 +69,7 @@ const std::string getResponse (const std::string& prompt)
   if (! line_read)
   {
     std::cout << "\n";
+    response = "<EOF>";
   }
   else
   {
@@ -83,6 +86,7 @@ const std::string getResponse (const std::string& prompt)
   if (std::cin.eof () == 1)
   {
     std::cout << "\n";
+    response = "<EOF>";
   }
 #endif
 
@@ -93,22 +97,27 @@ const std::string getResponse (const std::string& prompt)
 static int commandLoop (bool autoClear)
 {
   // Compose the prompt.
-  std::string prompt = promptCompose ();
+  auto prompt = promptCompose ();
 
   // Display prompt, get input.
-  std::string command = getResponse (prompt);
+  auto command = getResponse (prompt);
 
   // Obey Taskwarrior's rc.tasksh.autoclear.
   if (autoClear)
     std::cout << "\033[2J\033[0;0H";
 
   int status = 0;
-  if (command != "")
+  if (! isatty (fileno (stdin)) && command == "")
   {
-    std::vector <std::string> args = split (command, ' ');
+    status = -1;
+  }
+  else if (command != "")
+  {
+    auto args = split (command, ' ');
 
     // Dispatch command.
-         if (closeEnough ("exit",        args[0], 3)) status = -1;
+         if (args[0] == "<EOF>")                      status = -1;
+    else if (closeEnough ("exit",        args[0], 3)) status = -1;
     else if (closeEnough ("quit",        args[0], 3)) status = -1;
     else if (closeEnough ("help",        args[0], 3)) status = cmdHelp ();
     else if (closeEnough ("diagnostics", args[0], 3)) status = cmdDiagnostics ();
@@ -155,7 +164,9 @@ int main (int argc, const char** argv)
                    output == "yes\n"  ||
                    output == "on\n");
 
-      welcome ();
+      if (isatty (fileno (stdin)))
+        welcome ();
+
       while ((status = commandLoop (autoClear)) == 0)
         ;
     }
